import json
import urllib.request

def request(action, **params):
    return {'action': action, 'params': params, 'version': 6}

def invoke(action, server, **params):
    requestJson = json.dumps(request(action, **params)).encode('utf-8')
    response = json.load(urllib.request.urlopen(urllib.request.Request(server, requestJson)))
    if len(response) != 2:
        raise Exception('response has an unexpected number of fields')
    if 'error' not in response:
        raise Exception('response is missing required error field')
    if 'result' not in response:
        raise Exception('response is missing required result field')
    if response['error'] is not None:
        raise Exception(response['error'])
    return response['result']

def getDeckList(server):
    result = invoke('deckNames', server)
    return result

def getNoteTypes(server):
    result = invoke('modelNames', server)
    return result

def getFields(server, name):
    result = invoke('modelFieldNames', server, modelName=name)
    return result

def addNote(server, content):
    result = invoke('addNote', server, note=content)
    return result


import sys
from PyQt6.QtWidgets import QApplication, QWidget, QGridLayout, QTextEdit, QPushButton, QLabel, QLineEdit, QMainWindow, QDialog, QCheckBox, QVBoxLayout
from PyQt6.QtCore import QObject, QTimer, pyqtSlot, QSettings
from wiktionaryparser import WiktionaryParser
from ssmtool.config import SettingsDialog
from os import path
from ssmtool.tools import addNote

import functools

@functools.lru_cache()
class GlobalObject(QObject):
    def __init__(self):
        super().__init__()
        self._events = {}

    def addEventListener(self, name, func):
        if name not in self._events:
            self._events[name] = [func]
        else:
            self._events[name].append(func)

    def dispatchEvent(self, name):
        functions = self._events.get(name, [])
        for func in functions:
            QTimer.singleShot(0, func)




class DictionaryWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("Simple Sentence Mining")
        self.setFixedSize(400, 500)
        self.widget = QWidget()
        self.settings = QSettings("FreeLanguageTools", "SimpleSentenceMining")
        self.initDictionary()
        self.setCentralWidget(self.widget)
        self.initWidgets()
        self.setupWidgets()
        GlobalObject().addEventListener("double clicked", self.getDefinition)
        QApplication.clipboard().dataChanged.connect(self.clipboardChanged)
        
    def initWidgets(self):
        self.sentence = MyTextEdit("Sentence here")
        self.sentence.setMinimumHeight(30)
        self.sentence.setMaximumHeight(120)
        self.word = QLineEdit("Word here")
        self.definition = MyTextEdit("Definition here")
        self.definition.setMinimumHeight(70)
        self.definition.setMaximumHeight(400)
        self.label_sentence = QLabel("Sentence")
        self.label_word = QLabel("Word")
        self.label_def = QLabel("Definition")
        self.lookup_button = QPushButton("Get Definition")
        self.toanki_button = QPushButton("Add note")
        self.config_button = QPushButton("Configure..")
    
    def setupWidgets(self):
        self.layout = QGridLayout(self.widget)
        self.layout.addWidget(self.label_sentence, 0, 0, 1, 2)
        self.layout.addWidget(self.label_word, 2, 0, 1, 2)
        self.layout.addWidget(self.label_def, 4, 0, 1, 2)

        self.layout.addWidget(self.sentence, 1, 0, 1, 2)
        self.layout.addWidget(self.word, 3, 0, 1, 2)
        self.layout.addWidget(self.definition, 5, 0, 1, 2)

        self.layout.addWidget(self.lookup_button, 6, 0)
        self.layout.addWidget(self.toanki_button, 6, 1)
        self.layout.addWidget(self.config_button, 7, 0, 1, 2)

        self.lookup_button.clicked.connect(self.getDefinition)
        self.config_button.clicked.connect(self.configure)
        self.toanki_button.clicked.connect(self.createNote)

    def initDictionary(self):
        self.parser = WiktionaryParser()
        #self.parser.set_default_language(self.config['DEFAULT'].get("target_language"))
        #print("Target language is set to: " + self.parser.get_default_language())

    def configure(self):
        self.settings_dialog = SettingsDialog(self)
        self.settings_dialog.show()


    def getDefinition(self):
        result = ""
        cursor = self.sentence.textCursor()
        selected = cursor.selectedText().lower()
        cursor2 = self.definition.textCursor()
        selected2 = cursor2.selectedText().lower()
        if selected != "":
            result = self.lookup(selected)
            self.word.setText(selected)
            self.definition.setText(result)
            cursor.clearSelection()
            self.sentence.setTextCursor(cursor)
        elif selected2 != "":
            result = self.lookup(selected2)
            self.definition.setText(result)
            cursor.clearSelection()
            self.sentence.setTextCursor(cursor)
            #cursor2.clearSelection()
        else:
            return
        
    
    def setSentence(self, content):
        self.sentence.setText(content)

    def clipboardChanged(self):
        text = QApplication.clipboard().text()
        self.setSentence(text)

    def lookup(self, word):
        print("Looking up: ", word)
        item = self.parser.fetch(word)
        meanings = []
        for i in item:
            for j in i['definitions']:
                meanings.append("\n".join(j['text'][1:]))
        return word + "\n" + 30*"=" + "\n" + ("\n" + 30*"-" + "\n").join(meanings)

    def createNote(self):
        sentence = self.sentence.toPlainText()
        word = self.word.text()
        definition = self.definition.toPlainText()
        content = {
            "deckName": self.settings.value("deck_name"),
            "modelName": self.settings.value("note_type"),
            "fields": {
                self.settings.value("sentence_field"): sentence,
                self.settings.value("word_field"): word,
                self.settings.value("definition_field"): definition
            }
        }
        addNote(self.settings.value("anki_api"), content)


class MyTextEdit(QTextEdit):
    #def focusOutEvent(self, e):
    #    start = self.selectionStart()
    #    if start == -1:
    #        return
    #    length = self.selectionLength()
    #    super().focusOutEvent(e)
    #    self.setSelection(start, length)
    @pyqtSlot()
    def mouseDoubleClickEvent(self, e):
        super().mouseDoubleClickEvent(e)
        GlobalObject().dispatchEvent("double clicked")
        print("Event sent")
        self.textCursor().clearSelection()


if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = DictionaryWindow()

    #keybinder.register_hotkey(w.winId(), "Shift+Ctrl+A", lambda _:print("hello"))
    
    w.show()
    sys.exit(app.exec())

    
    



from PyQt6.QtWidgets import QDialog, QCheckBox, QGridLayout, QLineEdit, QComboBox, QLabel
from ssmtool.tools import getDeckList, getNoteTypes, getFields

class SettingsDialog(QDialog):
    def __init__(self, parent):
        super().__init__(parent)
        self.settings = parent.settings
        self.initWidgets()
        self.setupWidgets()
        self.loadSettings()
        self.setupAutosave()

    def initWidgets(self):
        self.layout = QGridLayout(self)

        self.allow_editing = QCheckBox("Allow directly editing")
        self.lemmatization = QCheckBox("Use lemmatization (NOT IMPLEMENTED)")
        self.target_language = QComboBox()
        self.deck_name = QComboBox()
        self.note_type = QComboBox()
        self.sentence_field = QComboBox()
        self.word_field = QComboBox()
        self.definition_field = QComboBox()
        self.anki_api = QLineEdit()

    def setupWidgets(self):
        languages = [
            "english",
            "french",
            "spanish",
            "russian",
            "japanese",
            "italian",
            "chinese"
        ]
        self.target_language.addItems(languages)
        self.layout.addWidget(self.allow_editing, 0, 0, 1, 2)
        self.layout.addWidget(self.lemmatization, 1, 0, 1, 2)
        self.lemmatization.setCheckable(False)

        self.layout.addWidget(QLabel("Target language"), 2, 0)
        self.layout.addWidget(self.target_language, 2, 1)

        self.layout.addWidget(QLabel('API endpoint for AnkiConnect'), 3, 0)
        self.layout.addWidget(self.anki_api, 3,1)

        self.layout.addWidget(QLabel("Deck name"), 4, 0)
        self.layout.addWidget(self.deck_name, 4, 1)

        self.layout.addWidget(QLabel("Note type"), 5, 0)
        self.layout.addWidget(self.note_type, 5,1)

        self.layout.addWidget(QLabel('Field name for "Sentence"'), 6, 0)
        self.layout.addWidget(self.sentence_field, 6, 1)

        self.layout.addWidget(QLabel('Field name for "Word"'), 7, 0)
        self.layout.addWidget(self.word_field, 7, 1)

        self.layout.addWidget(QLabel('Field name for "Definition"'), 8, 0)
        self.layout.addWidget(self.definition_field, 8, 1)


        
    def setupAutosave(self):
        self.allow_editing.clicked.connect(self.syncSettings)
        self.lemmatization.clicked.connect(self.syncSettings)
        self.target_language.currentTextChanged.connect(self.syncSettings)
        self.deck_name.currentTextChanged.connect(self.syncSettings)
        self.note_type.currentTextChanged.connect(self.syncSettings)
        self.note_type.currentTextChanged.connect(self.loadFields)
        self.sentence_field.currentTextChanged.connect(self.syncSettings)
        self.word_field.currentTextChanged.connect(self.syncSettings)
        self.definition_field.currentTextChanged.connect(self.syncSettings)
        self.anki_api.editingFinished.connect(self.syncSettings)

    def loadSettings(self):
        print("loading")
        self.allow_editing.setChecked(self.settings.value("allow_editing", True, type=bool))
        self.lemmatization.setChecked(self.settings.value("lemmatization", type=bool))
        self.target_language.setCurrentText(self.settings.value("target_language"))
        self.anki_api.setText(self.settings.value("anki_api", "http://localhost:8765"))

        api = self.anki_api.text()

        decks = getDeckList(api)
        self.deck_name.clear()
        self.deck_name.addItems(decks)
        self.deck_name.setCurrentText(self.settings.value("deck_name"))

        note_types = getNoteTypes(api)
        self.note_type.clear()
        self.note_type.addItems(note_types)
        self.note_type.setCurrentText(self.settings.value("note_type"))
        self.loadFields()

    def loadFields(self):
        print("loading fields")
        api = self.anki_api.text()
        current_type = self.note_type.currentText()
        fields = getFields(api, current_type)

        self.sentence_field.clear()
        self.sentence_field.addItems(fields)
        self.word_field.clear()
        self.word_field.addItems(fields)
        self.definition_field.clear()
        self.definition_field.addItems(fields)
        
        self.sentence_field.setCurrentText(self.settings.value("sentence_field"))
        self.word_field.setCurrentText(self.settings.value("word_field"))
        self.definition_field.setCurrentText(self.settings.value("definition_field"))


    def syncSettings(self):
        print("syncing")
        self.settings.setValue("allow_editing", self.allow_editing.isChecked())
        self.settings.setValue("lemmatization", self.lemmatization.isChecked())
        self.settings.setValue("target_language", self.target_language.currentText())
        self.settings.setValue("deck_name", self.deck_name.currentText())
        self.settings.setValue("note_type", self.note_type.currentText())
        self.settings.setValue("sentence_field", self.sentence_field.currentText())
        self.settings.setValue("word_field", self.word_field.currentText())
        self.settings.setValue("definition_field", self.definition_field.currentText())
        self.settings.setValue("anki_api", self.anki_api.text())
        self.settings.sync()